#ifndef ELMCIE_H
#define ELMCIE_H

//****************************** Start of user customizing area *******************************
// Calibration capacitors: The accuracy of these define the accuracy of the instrument

// Enter here the value of YOUR Elm calibration capacitor in pF's
#define C_cal1	100800

// Enter here the value of YOUR Elcie's cal cap : but in 0.1 pF's !!!
#define C_cal2	14990

// Enter here the 6 values you wrote down during Pre-flights 1, 2 and 3
#define Pre_flight_elm_null_cnts 5649			// counts
#define Pre_flight_elm_low_range_cal 31.6658	// counts per pF
#define Pre_flight_elm_high_range_cal 0.0309	// counts per pF

// Next two values are rounded by Pre_flight3
// This rounding does not influence the accuracy of Elmcie : it's used for setting limits only
#define Pre_flight_elcie_f_null 536684			// Elcie's base-frequency determined by C1 and L1
#define Pre_flight_l_elcie_null 87				// Inductor L1
#define Pre_flight_c_elcie_null 1006				// Capacitor C1
//******************************* end of user customizing area ********************************

//*********************************** Calculation of limits ***********************************
// Limits are used to prevent that you accidently overwrite the calibration-values in eeprom
// Example: If you press NULL when there is a device connected, leave it connected, and the program
// will display an error-message, but will not overwrite your precious cal-values.

#define Elm_null_cnts_max (Pre_flight_elm_null_cnts + 750)
#define Elm_null_cnts_min (Pre_flight_elm_null_cnts - 750)

#define Elm_low_range_cal_max (Pre_flight_elm_low_range_cal + 2)
#define Elm_low_range_cal_min (Pre_flight_elm_low_range_cal - 2)

#define Elm_high_range_cal_max (Pre_flight_elm_high_range_cal + 0.002)
#define Elm_high_range_cal_min (Pre_flight_elm_high_range_cal - 0.002)

#define Elcie_f_null_max (Pre_flight_elcie_f_null + 10000)
#define Elcie_f_null_min (Pre_flight_elcie_f_null - 10000)

#define L_elcie_null_max (Pre_flight_l_elcie_null + 2)
#define L_elcie_null_min (Pre_flight_l_elcie_null - 2)

#define C_elcie_null_max (Pre_flight_c_elcie_null + 20)
#define C_elcie_null_min (Pre_flight_c_elcie_null - 20)

//******************************** End of calculation of limits *******************************

#define Factor2 2533		// 10^5/(4*pi*pi)

typedef struct {
  unsigned char bit0:1;
  unsigned char bit1:1;
  unsigned char bit2:1;
  unsigned char bit3:1;
  unsigned char bit4:1;
  unsigned char bit5:1;
  unsigned char bit6:1;
  unsigned char bit7:1;
} io_reg;

#define Elm_disch			( (volatile io_reg*)(&DDRD) )->bit6
#define Elm_low_threshold	( (volatile io_reg*)(&DDRD) )->bit4
#define N_elm3k3_lm311on	( (volatile io_reg*)(&PORTD) )->bit3
#define Do_l				( (volatile io_reg*)(&PORTB) )->bit0
#define Backlight_hi		( (volatile io_reg*)(&PORTB) )->bit1
#define Spkr				( (volatile io_reg*)(&PORTC) )->bit4

#define Status	GPIOR0

#define Elm_selected	( (volatile io_reg*)(&Status) )->bit7
#define Elm_large_caps	( (volatile io_reg*)(&Status) )->bit6
#define Do_nulling		( (volatile io_reg*)(&Status) )->bit5
#define Do_calibration	( (volatile io_reg*)(&Status) )->bit4
#define Elcie_cap		( (volatile io_reg*)(&Status) )->bit3
#define Elm_missed_it	( (volatile io_reg*)(&Status) )->bit2
#define Elm_timeout		( (volatile io_reg*)(&Status) )->bit1
#define Elm_ready		( (volatile io_reg*)(&Status) )->bit0


// Use unions to be able to access every byte or word from Elmcie and Elm_high
// In Nard's original program Elmcie = Cnts1 and Elm_high = Cnts2
volatile union {
 uint32_t Counts;
 uint16_t ints[2];
 uint8_t  bytes[4];
} Elmcie, Elm_high;

// Variable definitions
volatile uint16_t Cnt_10ms;
		char Teh_result_str[17];		// 16 columns + /0
		char Unit;
		float	Teh_result;


//uint8_t Status	// defined above to use GPIO
uint8_t Status_copy;

// Elm variables
volatile	uint32_t Elm_cnt;
volatile	uint32_t Elm_null_cnts;

	float	Elm_high_range_cal;
	float	Elm_low_range_cal;

// Elcie variables
	uint32_t Elcie_f_null;
	uint32_t Elcie_f_dut;
	
	float	L_elcie_dut;
	float	L_elcie_null;
	float	C_elcie_dut;
	float	C_elcie_null;

// Other variables
	uint32_t Tmp_long;
	uint16_t Tmp_word;
	uint8_t  Tmp_byte;
	float	 Tmp_single;


// EEPROM variables
float	 EEMEM Ee_elm_high_range_cal = Pre_flight_elm_high_range_cal;
float   EEMEM Ee_elm_low_range_cal = Pre_flight_elm_low_range_cal;
uint32_t EEMEM Ee_elm_null_cnts = Pre_flight_elm_null_cnts;
uint32_t EEMEM Ee_elcie_f_null = Pre_flight_elcie_f_null;
float   EEMEM Ee_c_elcie_null = Pre_flight_c_elcie_null;
float   EEMEM Ee_l_elcie_null = Pre_flight_l_elcie_null;


#endif
