//*****************************************************************************
//
// File Name	: 'keys.C'
// Title		: Debounce and repeat keys
// Author		: Peter Dannegger (code) Felipe Maimon (lib)
// Created		: 2009-03-06
// Target MCU	: Atmel AVR series
//
//
// Peter Dannegger's code to debounce and repeat keys. Up to 8 keys
// (at the same port) can be debounced at once. Just create an interrupt
// triggered every 10 ms and call the Read_Keys function from it.
//
//*****************************************************************************

#include <avr/io.h>
#include <avr/interrupt.h>
#include "keys.h"


u8 get_key_press( u8 key_mask )
{
  cli();					// read and clear atomic !
  key_mask &= key_press;                        // read key(s)
  key_press ^= key_mask;                        // clear key(s)
  sei();
  return key_mask;
}


u8 get_key_rpt( u8 key_mask )
{
  cli();					// read and clear atomic !
  key_mask &= key_rpt;                        	// read key(s)
  key_rpt ^= key_mask;                        	// clear key(s)
  sei();
  return key_mask;
}


u8 get_key_short( u8 key_mask )
{
  cli();			// read key state and key press atomic !
  return get_key_press( ~key_state & key_mask );
}


u8 get_key_long( u8 key_mask )
{
  return get_key_press( get_key_rpt( key_mask ));
}
